/**
 * \file psa/build_info.h
 *
 * \brief Build-time PSA configuration info
 *
 *  Include this file if you need to depend on the
 *  configuration options defined in mbedtls_config.h or MBEDTLS_CONFIG_FILE
 *  in PSA cryptography core specific files.
 */
/*
 *  Copyright The Mbed TLS Contributors
 *  SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 */

#ifndef PSA_CRYPTO_BUILD_INFO_H
#define PSA_CRYPTO_BUILD_INFO_H

#include "mbedtls/build_info.h"

#endif /* PSA_CRYPTO_BUILD_INFO_H */
